 /*-------------------------------------------------------------------*
  *    Name: sunplot.sas                                              *
  *   Title: Sunflower plot for X-Y dataset                           *
  *     Doc: http://www.math.yorku.ca/SCS/sasmac/sunplot.html         *
  *                                                                   *
  *-------------------------------------------------------------------*
  *  Author:  Michael Friendly            <friendly@yorku.ca>         *
  * Created:  10 Jul 1996 10:26:49                                    *
  * Revised:   4 Feb 1997 15:31:35                                    *
  * Version:  1.1                                                     *
  *      From ``SAS System for Statistical Graphics, First Edition''  *
  *      Copyright(c) 1991 by Michael Friendly                        *
  * - transfer variable labels to plotting variables                  *
  *-------------------------------------------------------------------*/

 /*=
The sunflower plot displays a bivariate dataset using "sunflower
symbols" to show the number of observations in the neighborhood
of each XY point.

 =*/
 
%*-- Requires that the program sunfont.sas has been previously run to
	create the sunflower 'sun' font, stored in the libname designated
	by gfont0;
	
libname gfont0 '~/sasuser/gfont';

%macro sunplot(
	data=_LAST_,   /* name of the input dataset              */
	x=x,           /* name of the horizontal variable        */
	y=y,           /* name of the vertical variable          */
	nx=20,         /* number of slices of the horizontal var */
	ny=20,         /* number of slices of the vertical var   */
	color=black,   /* color for plotting symbols             */
	sunfont=sun,
	hsym=1.5,      /* symbol height                          */
	vaxis=,        /* optional name of axis statement        */
	haxis=,        /* optional name of axis statement        */
	plotopt=nolegend frame,  /* additional plot options */
	anno=,
	name=sun       /* nsme of graphics catalog entry    */
	);
	
	%global gout;
	%if &x=%str() or &y=%str() %then %do;
		%put SUNPLOT: X= and Y= variables must be specified;
		%goto DONE;
		%end;
 

	%cell(data=&data, x=&x, y=&y, nx=&nx, ny=&ny, out=_binned_);
	%sunsym(data=_binned_, ht=&hsym, sunfont=&sunfont);
	
	proc gplot data=_binned_ &GOUT ;
   plot yc * xc = count /
         &plotopt
			%if %length(&vaxis)>0 %then vaxis=&vaxis ;
			%if %length(&vaxis)>0 %then haxis=&haxis ;
			%if %length(&anno)>0 %then anno=&anno ;
         name="&name"
			des="sunflower plot of &data (&y * &x)" ;
		run; quit;
%done:
goptions reset=symbol;
%mend;


 /*------------------------------------*
  | Macro to cellulate an X-Y data set |
  *------------------------------------*/
%macro cell( data=_LAST,     /* input data set */
             out=_DATA_,
             x=X, y=Y,       /* input X, Y variables */
             xc=xc, yc=yc,   /* output X, Y variables */
             nx=20, ny=20);  /* number of bins in X & Y directions */
 
 /*-------------------------------------------------*
  | Quantize the X and Y values into discrete cells |
  *-------------------------------------------------*/
proc means data=&data noprint;
   var &x &y;
   output out=_temp_ range=rx ry;
 
proc contents noprint data=&data out=_work_(keep=name type label);

data _null_;
   set _work_;
	if name=upcase("&x") then do;
		if label ^= ' ' then call symput('lx', trim(label));
		else call symput('lx', "&x");
		end;
	if name=upcase("&y") then do;
		if label ^= ' ' then call symput('ly', trim(label));
		else call symput('ly', "&y");
		end;
	
data _cell_;
   if _n_=1 then set _temp_;
   set &data;
   drop rx ry;
   deltax = rx / &nx;
   deltay = ry / &nx;
   &xc = deltax * (round( &x/deltax )+.5);
   &yc = deltay * (round( &y/deltay )+.5);
run;
%*put lx=&lx ly=&ly;
 /*----------------------------------------*
  | Find number of points in each X-Y cell |
  *----------------------------------------*/
proc freq data=_cell_ ;
   tables &xc * &yc / noprint out=&out ;
	run;
  *-- assign variable labels to plotting variables;
data &out;
	set &out;
	label xc = "&lx"
	      yc = "&ly";
run;	
%mend;
 /*:
 :*/
 /*----------------------------------------------------*
  |  Macro to generate SYMBOL statement for each COUNT |
  *----------------------------------------------------*/
%macro sunsym( data=_LAST_,ht=1.5, color=black, sunfont=sun );
%local alpha ch repeat;
%let alpha=ABCDEFGHIJKLMNOPQRSTUVWXYZ;
 
proc means data=&data noprint;
   var count;
   output out=_temp_ max=maxcount;
data _null_;
   set _temp_;
   maxcount = min( maxcount, 26 );
   call symput('HIGH',put(maxcount,2.));
run;
 
  %let repeat=;
  %do i=1 %to &HIGH %by 1;
     %let ch = &i;
     %if &i = 26 %then %let repeat = r=100;
     symbol&i h=&ht f=&sunfont c=&color v=%substr(&alpha,&ch,1) &repeat ;
  %end;
%mend;
