 /*--------------------------------------------------------------*
  *    Name: symbox.sas                                          *
  *   Title: Show transformations of a variable as boxplots      *
  *     Doc: http://www.math.yorku.ca/SCS/sasmac/symbox.html     *
  *--------------------------------------------------------------*
  *  Author:  Michael Friendly            <friendly@yorku.ca>    *
  * Created:  24 Sep 1997 06:55:08                               *
  * Revised:  20 May 2000 14:02:25                               *
  * Version: 1.0                                                 *
  * 1.1  - Use variable label in the boxplot; note dropped obs.  *
  *                                                              *
  *--------------------------------------------------------------*/
 /*=
=Description:
 
 Various graphical displays for diagnosing symmetry of a distribution
 are available in the SYMPLOT macro.  SYMBOX takes a more direct
 approach.  The variable is first transformed to each of the selected
 powers, then each transform is standardized to mean=0, std=1, and
 then displayed as side-by-side boxplots.
	
 Missing and non-positive observations are excluded.
	
=Usage:
 
 =*/
 
%macro symbox(
	data=_last_,     /* name of input data set */
	var=,            /* name of variable to examine  */
	id=,             /* name of ID variable */
	out=symout,      /* name of output data set */
	orient=V,        /* orientation of boxplots */
	powers=-1 -0.5 0 .5 1,  /* list of powers to consider */
	name=symbox
	);

%local i npow;
%let npow = %numwords(&powers);

%let p1 = %scan(&powers,1,%str( ));
%let p2 = %scan(&powers,&npow,%str( ));
%*put npow = &npow p1=&p1 p2=&p2 p3=&p3;

proc format;
   value pow -1 ='-1/X' -0.5= '-1/Sqrt' 0='Log' .5='Sqrt' 1='Raw';
*options mprint symbolgen;

data _null_;
	set &data(obs=1);
	length _label_ $40;
	call label(&var, _label_);
	_label_= trim(_label_) || ' (Std.)';
	call symput('label', _label_);
	run;
%put VAR=&var LABEL=&LABEL;
*options nomprint nosymbolgen;

data _bb_;
   set &data end=eof;
   keep var power &id;
   label var="&label";

   if &var > 0 then do;
      %do i=1 %to &npow;
         power = %scan(&powers, &i, %str( ));
			if power=0 then var = log10(&var);
			           else var =  (&var)**power / power;
      output;
      %end;
   end;
	else do;
		dropped+1;
		end;
	if eof then do;
		if dropped>0 
			then put 'NOTE: ' dropped ' observatations were missing or negative, and were excluded from the analysis.';
		end;
run;

proc sort; 
   by power;
proc standard m=0 s=1 out=&out;
   var var;
   by power;
run;

%boxplot(data=&out, var=var, id=&id,
    class=Power, classfmt=pow., orient=&orient, name=&name,
    xorder=&P1 to &P2 by .5, varlab=&label, print=);
%mend;

%macro numwords(lst);
   %local i;
   %let i = 1;
   %let v = %scan(&lst,&i);
   %do %while (%length(&v) > 0);
      %let i = %eval(&i + 1);
      %let v = %scan(&lst,&i, %str( ));
      %end;
   %eval(&i - 1)
%mend;
